/***************************************************************************
 *  GBAGI: The Game Boy Advance Adventure Game Interpreter
 *  Copyright (C) 2003,2004 Brian Provinciano, http://www.bripro.com
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ***************************************************************************/

/*****************************************************************************/
#include "gbagi.h"
#include "input.h"
#include "views.h"
#include "system.h"
#include "parse.h"
#include "text.h"
#include "wingui.h"
/*****************************************************************************/
EVENT tmpEvent, evStopEgo = {EV_DIRECTION, dirNONE};
BTNSTATE btnstate;
/*****************************************************************************/
const KEY keyDirs[]={
	{KEY_UP, 		dirUP		}, {KEY_PGUP,		dirUPRIGHT		},
    {KEY_RIGHT, 	dirRIGHT	}, {KEY_PGDOWN, 	dirDOWNRIGHT	},
    {KEY_DOWN, 		dirDOWN		}, {KEY_END, 		dirDOWNLEFT		},
    {KEY_LEFT, 		dirLEFT		}, {KEY_HOME, 		dirUPLEFT		},
    {KEY_NUMPAD8,	dirUP		}, {KEY_NUMPAD9, 	dirUPRIGHT		},
    {KEY_NUMPAD6, 	dirRIGHT	}, {KEY_NUMPAD3, 	dirDOWNRIGHT	},
    {KEY_NUMPAD2, 	dirDOWN		}, {KEY_NUMPAD1, 	dirDOWNLEFT		},
    {KEY_NUMPAD4, 	dirLEFT		}, {KEY_NUMPAD7, 	dirUPLEFT		},
    {0,0}
};
/*****************************************************************************/
#ifdef _WINDOWS
int DelayTimes[5] = {0,0,0,0,0};//{0,12,24,32,60};//{0,12,24,40};
#else
int DelayTimes[5] = {0,96,140,280,560};//{0,32,96,320};
#endif
void DoDelayNPoll()
{
	Delay(-1);
	PollInput();
}
/*****************************************************************************/
void Delay(int amt)
{
	int delay;
#ifdef _WINDOWS    
	if(amt==-1)
		amt = (DelayTimes[vars[vDELAY]<4?vars[vDELAY]:4]);
	while(amt>>4) {
    	SystemUpdate();
    	amt--;
    	PollInput();
    }
#else
	if(amt==-1) {
     	delay = (DelayTimes[vars[vDELAY]<4?vars[vDELAY]:4])*10+1;
		while(REG_TM1D <= delay){
			PollInput();
        }
        REG_TM1CNT=0;
		REG_TM1D = 0;
		REG_TM1CNT = TIME_FREQUENcy1024 | TIME_ENABLE;
    } else {
		delay = (64*amt)+1;
		//Start the timer
		REG_TM0CNT = TIME_FREQUENcy1024 | TIME_ENABLE;
		REG_TM0D = 0;
		while(REG_TM0D <= delay){
			//PollInput();
        }
		REG_TM0CNT = 0;
    }
#endif
}
/*****************************************************************************/
void PollInput()
{
	EVENT *event;   
	CTLMAP *c;

    while((BOOL)(event = ReadEvent()) && (!TestFlag(fPLAYERCOMMAND))) {
		if(event->type==EV_DIRECTION) {
        	vars[vEGODIR] =
                (WALK_HOLD || (event->data != ViewObjs[0].direction))? event->data : 0;
            if(PLAYER_CONTROL)
				ViewObjs[0].motion = mtNONE;
        } else {
			for(c = ctlMap; c < ctlMap+MAX_CONTROLLERS; c++)
				if(event->data == c->key) {
                	controllers[c->num]=1;
					break;
				}
        	vars[vKEYPRESSED] = event->data;
            if(!GUI_ACTIVE){
            	if(INPUT_ENABLED&&event->data==KEY_ENTER)
                	ExecuteInputDialog(TRUE);
                /*if(event->data=='`') {
                	MessageBox("GBAGI Menu\n  (here)");
                } */
            }
		}
	}
}
/*****************************************************************************/
EVENT *ReadEvent(void)
{                   
	KEY *k = (KEY*)keyDirs;
    int key=SystemCheckKey();
    if(key) {
		while(k->symbol && (k->symbol != key)) k++;
		if(k->symbol) {
			tmpEvent.type = EV_DIRECTION;
			tmpEvent.data = k->value;
		} else {
			tmpEvent.type = EV_ASCII;
			tmpEvent.data = key;
		}
    	return &tmpEvent;
    } else
    	if(WALK_HOLD && btnstate.state==BTN_RELEASE)
			return &evStopEgo;
	return NULL;
}
/*****************************************************************************/
int PollKey()
{
	EVENT *event;
	if((BOOL)(event = ReadEvent()))
		return (event->type == EV_ASCII)? event->data : -1;
   	return 0;
}
/*****************************************************************************/
int CheckUserReply()
{
	switch(PollKey()) {
		case KEY_ENTER:
			return 1;
		case KEY_ESC:
			return 0;
	}
	return -1;
}
/*****************************************************************************/
int WaitForKey()
{
	int key;
    while((!(key = PollKey())) || (key == -1))
        Delay(5);
    return key;
}
/*****************************************************************************/
BOOL WaitEnterEsc()
{
	int key;
	while((key=CheckUserReply()) == -1)
		Delay(5);
	return key;
}
/*****************************************************************************/
EVENT *WaitForEvent()
{
	EVENT *event;
	while(!(BOOL)(event = ReadEvent()))
		Delay(5);
	return event;
}
/*****************************************************************************/

